@@ -10,25 +10,7 @@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
-* as published by the Free Software Foundation with the following additional
-* term according to sec. 7:
-*
-* According to sec. 7 of the GNU Affero General Public License, version 3,
-* the terms of the AGPL are supplemented with the following terms:
-*
-* "Zarafa" is a registered trademark of Zarafa B.V.
-* "Z-Push" is a registered trademark of Zarafa Deutschland GmbH
-* The licensing of the Program under the AGPL does not imply a trademark license.
-* Therefore any rights, title and interest in our trademarks remain entirely with us.
-*
-* However, if you propagate an unmodified version of the Program you are
-* allowed to use the term "Z-Push" to indicate that you distribute the Program.
-* Furthermore you may use our trademarks where it is necessary to indicate
-* the intended purpose of a product or service provided you use it in accordance
-* with honest practices in industrial or commercial matters.
-* If you want to propagate modified versions of the Program under the name "Z-Push",
-* you may only do so if you have a written permission by Zarafa Deutschland GmbH
-* (to acquire a permission please contact Zarafa at trademark@zarafa.com).
+* as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -200,6 +182,11 @@ class Sync extends RequestProcessor {
                         }
                     }
 
+                    // determine if this is the KOE GAB folder so it can be prioritized by SyncCollections
+                    if (KOE_CAPABILITY_GAB && self::$deviceManager->IsKoe() && $spa->GetBackendFolderId() == self::$deviceManager->GetKoeGabBackendFolderId()) {
+                        $spa->SetKoeGabFolder(true);
+                    }
+
                     // done basic SPA initialization/loading -> add to SyncCollection
                     $sc->AddCollection($spa);
                     $sc->AddParameter($spa, "requested", true);
@@ -398,7 +385,7 @@ class Sync extends RequestProcessor {
                     }
 
                     // unset filtertype for KOE GAB folder
-                    if (KOE_CAPABILITY_GAB && self::$deviceManager->IsKoe() && $spa->GetBackendFolderId() == self::$deviceManager->GetKoeGabBackendFolderId()) {
+                    if ($spa->GetKoeGabFolder() === true) {
                         $spa->SetFilterType(SYNC_FILTERTYPE_ALL);
                         ZLog::Write(LOGLEVEL_DEBUG, "HandleSync(): KOE GAB folder - setting filter type to unlimited");
                     }
@@ -705,6 +692,9 @@ class Sync extends RequestProcessor {
                     }
                 }
 
+                // update the waittime waited
+                self::$waitTime = $sc->GetWaitedSeconds();
+
                 // in case there are no changes and no other request has synchronized while we waited, we can reply with an empty response
                 if (!$foundchanges && $status == SYNC_STATUS_SUCCESS) {
                     // if there were changes to the SPA or CPOs we need to save this before we terminate
@@ -747,6 +737,7 @@ class Sync extends RequestProcessor {
         // global status
         // SYNC_COMMONSTATUS_* start with values from 101
         if ($status != SYNC_COMMONSTATUS_SUCCESS && ($status == SYNC_STATUS_FOLDERHIERARCHYCHANGED || $status > 100)) {
+            self::$deviceManager->AnnounceProcessStatus($folderid, $status);
             $this->sendStartTags();
             self::$encoder->startTag(SYNC_STATUS);
                 self::$encoder->content($status);
@@ -765,8 +756,12 @@ class Sync extends RequestProcessor {
                 continue;
             }
 
-            if (! $sc->GetParameter($spa, "requested"))
+            if (! $sc->GetParameter($spa, "requested")) {
                 ZLog::Write(LOGLEVEL_DEBUG, sprintf("HandleSync(): partial sync for folder class '%s' with id '%s'", $spa->GetContentClass(), $spa->GetFolderId()));
+                // reload state and initialize StateMachine correctly
+                $sc->AddParameter($spa, "state", null);
+                $status = $this->loadStates($sc, $spa, $actiondata);
+            }
 
             // initialize exporter to get changecount
             $changecount = false;
@@ -778,8 +773,8 @@ class Sync extends RequestProcessor {
             // TODO we could check against $sc->GetChangedFolderIds() on heartbeat so we do not need to configure all exporter again
             if($status == SYNC_STATUS_SUCCESS && ($sc->GetParameter($spa, "getchanges") || ! $spa->HasSyncKey())) {
 
-                // no need to run the exporter if the globalwindowsize is already full
-                if ($sc->GetGlobalWindowSize() == $this->globallyExportedItems) {
+                // no need to run the exporter if the globalwindowsize is already full - if collection already has a synckey (ZP-1215)
+                if ($sc->GetGlobalWindowSize() == $this->globallyExportedItems && $spa->HasSyncKey()) {
                     ZLog::Write(LOGLEVEL_DEBUG, sprintf("Sync(): no exporter setup for '%s' as GlobalWindowSize is full.", $spa->GetFolderId()));
                     $setupExporter = false;
                 }
@@ -789,6 +784,12 @@ class Sync extends RequestProcessor {
                     $setupExporter = false;
                 }
 
+                // ZP-907: never send changes of UNKNOWN folders to an Outlook client
+                if (Request::IsOutlook() && self::$deviceManager->GetFolderTypeFromCacheById($spa->GetFolderId()) == SYNC_FOLDER_TYPE_UNKNOWN && $spa->HasSyncKey()) {
+                    ZLog::Write(LOGLEVEL_DEBUG, sprintf("Sync(): no exporter setup for '%s' as type is UNKNOWN.", $spa->GetFolderId()));
+                    $setupExporter = false;
+                }
+
                 // compare the folder statistics if the backend supports this
                 if ($setupExporter && self::$backend->HasFolderStats()) {
                     // check if the folder stats changed -> if not, don't setup the exporter, there are no changes!
@@ -854,6 +855,7 @@ class Sync extends RequestProcessor {
                             self::$topCollector->AnnounceInformation(sprintf("StatusException code: %d", $status), $this->singleFolder);
                             $this->saveMultiFolderInfo("exception", "StatusException");
                         }
+                        self::$deviceManager->AnnounceProcessStatus($spa->GetFolderId(), $status);
                     }
                 }
             }
@@ -914,8 +916,12 @@ class Sync extends RequestProcessor {
         // final top announcement for a multi-folder sync
         if ($sc->GetCollectionCount() > 1) {
             self::$topCollector->AnnounceInformation($this->getMultiFolderInfoLine($sc->GetCollectionCount()), true);
+            ZLog::Write(LOGLEVEL_DEBUG, sprintf("HandleSync: Processed %d folders", $sc->GetCollectionCount()));
         }
 
+        // update the waittime waited
+        self::$waitTime = $sc->GetWaitedSeconds();
+
         return true;
     }
 
@@ -1582,6 +1588,8 @@ class Sync extends RequestProcessor {
 
         $interval = Utils::GetFiltertypeInterval($spa->GetFilterType());
         $timeout = time() + (($interval && $interval < $maxTimeout) ? $interval : $maxTimeout);
+        // randomize timout in 12h
+        $timeout -= rand(0, 43200);
         ZLog::Write(LOGLEVEL_DEBUG, sprintf("Sync()->setFolderStat() on %s: %s expiring %s", $spa->getFolderId(), $newFolderStat, date('Y-m-d H:i:s', $timeout)));
         $spa->SetFolderStatTimeout($timeout);
     }
