@@ -10,25 +10,7 @@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
-* as published by the Free Software Foundation with the following additional
-* term according to sec. 7:
-*
-* According to sec. 7 of the GNU Affero General Public License, version 3,
-* the terms of the AGPL are supplemented with the following terms:
-*
-* "Zarafa" is a registered trademark of Zarafa B.V.
-* "Z-Push" is a registered trademark of Zarafa Deutschland GmbH
-* The licensing of the Program under the AGPL does not imply a trademark license.
-* Therefore any rights, title and interest in our trademarks remain entirely with us.
-*
-* However, if you propagate an unmodified version of the Program you are
-* allowed to use the term "Z-Push" to indicate that you distribute the Program.
-* Furthermore you may use our trademarks where it is necessary to indicate
-* the intended purpose of a product or service provided you use it in accordance
-* with honest practices in industrial or commercial matters.
-* If you want to propagate modified versions of the Program under the name "Z-Push",
-* you may only do so if you have a written permission by Zarafa Deutschland GmbH
-* (to acquire a permission please contact Zarafa at trademark@zarafa.com).
+* as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -219,19 +201,17 @@ class Utils {
      * @return boolean installed version is superior to the checked string
      */
     static public function CheckMapiExtVersion($version = "") {
+        if (!extension_loaded("mapi")) {
+            return false;
+        }
         // compare build number if requested
         if (preg_match('/^\d+$/', $version) && strlen($version) > 3) {
             $vs = preg_split('/-/', phpversion("mapi"));
             return ($version <= $vs[1]);
         }
-
-        if (extension_loaded("mapi")){
-            if (version_compare(phpversion("mapi"), $version) == -1){
-                return false;
-            }
-        }
-        else
+        if (version_compare(phpversion("mapi"), $version) == -1){
             return false;
+        }
 
         return true;
     }
@@ -408,13 +388,15 @@ class Utils {
         // make sure length is always an interger
         $length = (int)$length;
 
-        if (strlen($string) <= $length)
-            return $string;
+        // if the input string is shorter then the trunction, make sure it's valid UTF-8!
+        if (strlen($string) <= $length) {
+            $length = strlen($string) - 1;
+        }
 
         while($length >= 0) {
-            if ((ord($string[$length]) < 0x80) || (ord($string[$length]) >= 0xC0))
+            if ((ord($string[$length]) < 0x80) || (ord($string[$length]) >= 0xC0)) {
                 return substr($string, 0, $length);
-
+            }
             $length--;
         }
         return "";
@@ -530,6 +512,21 @@ class Utils {
         return $string;
     }
 
+    /**
+     * Converts an UTF7-IMAP encoded string into an UTF-8 string.
+     *
+     * @param string $string to convert
+     *
+     * @access public
+     * @return string
+     */
+    static public function Utf7imap_to_utf8($string) {
+        if (function_exists("mb_convert_encoding")){
+            return @mb_convert_encoding($string, "UTF-8", "UTF7-IMAP");
+        }
+        return $string;
+    }
+
     /**
      * Converts an UTF-8 encoded string into an UTF-7 string.
      *
@@ -548,6 +545,21 @@ class Utils {
         return $string;
     }
 
+    /**
+     * Converts an UTF-8 encoded string into an UTF7-IMAP string.
+     *
+     * @param string $string to convert
+     *
+     * @access public
+     * @return string
+     */
+    static public function Utf8_to_utf7imap($string) {
+        if (function_exists("mb_convert_encoding")){
+            return @mb_convert_encoding($string, "UTF7-IMAP", "UTF-8");
+        }
+        return $string;
+    }
+
     /**
      * Checks for valid email addresses
      * The used regex actually only checks if a valid email address is part of the submitted string
@@ -574,52 +586,6 @@ class Utils {
         return (bool) preg_match("#^([A-Za-z0-9+/]{4})*([A-Za-z0-9+/]{2}==|[A-Za-z0-9+\/]{3}=|[A-Za-z0-9+/]{4})?$#", $string);
     }
 
-    /**
-     * Decodes base64 encoded query parameters. Based on dw2412 contribution.
-     *
-     * @param string $query     the query to decode
-     *
-     * @access public
-     * @return array
-     */
-    static public function DecodeBase64URI($query) {
-        /*
-         * The query string has a following structure. Number in () is position:
-         * 1 byte       - protocoll version (0)
-         * 1 byte       - command code (1)
-         * 2 bytes      - locale (2)
-         * 1 byte       - device ID length (4)
-         * variable     - device ID (4+device ID length)
-         * 1 byte       - policy key length (5+device ID length)
-         * 0 or 4 bytes - policy key (5+device ID length + policy key length)
-         * 1 byte       - device type length (6+device ID length + policy key length)
-         * variable     - device type (6+device ID length + policy key length + device type length)
-         * variable     - command parameters, array which consists of:
-         *                      1 byte      - tag
-         *                      1 byte      - length
-         *                      variable    - value of the parameter
-         *
-         */
-        $decoded = base64_decode($query);
-        $devIdLength = ord($decoded[4]); //device ID length
-        $polKeyLength = ord($decoded[5+$devIdLength]); //policy key length
-        $devTypeLength = ord($decoded[6+$devIdLength+$polKeyLength]); //device type length
-        //unpack the decoded query string values
-        $unpackedQuery = unpack("CProtVer/CCommand/vLocale/CDevIDLen/H".($devIdLength*2)."DevID/CPolKeyLen".($polKeyLength == 4 ? "/VPolKey" : "")."/CDevTypeLen/A".($devTypeLength)."DevType", $decoded);
-
-        //get the command parameters
-        $pos = 7 + $devIdLength + $polKeyLength + $devTypeLength;
-        $decoded = substr($decoded, $pos);
-        while (strlen($decoded) > 0) {
-            $paramLength = ord($decoded[1]);
-            $unpackedParam = unpack("CParamTag/CParamLength/A".$paramLength."ParamValue", $decoded);
-            $unpackedQuery[ord($decoded[0])] = $unpackedParam['ParamValue'];
-            //remove parameter from decoded query string
-            $decoded = substr($decoded, 2 + $paramLength);
-        }
-        return $unpackedQuery;
-    }
-
     /**
      * Returns a command string for a given command code.
      *
@@ -1183,6 +1149,49 @@ class Utils {
 
         return $data;
     }
+
+    /**
+     * Modifies a SyncFolder object, changing the type to SYNC_FOLDER_TYPE_UNKNOWN but saving the original type.
+     * It also appends a zero-width UTF-8 (U+200B) character to the name, which serves as marker.
+     *
+     * @access public
+     * @param SyncFolder $folder
+     * @return SyncFolder
+     */
+    public static function ChangeFolderToTypeUnknownForKoe($folder) {
+        // append a zero width UTF-8 space to the name
+        $folder->displayname .= hex2bin("e2808b");
+        $folder->TypeReal = $folder->type;
+        $folder->type = SYNC_FOLDER_TYPE_UNKNOWN;
+
+        return $folder;
+    }
+
+    /**
+     * Checks if the displayname of the folder contains the zero-width UTF-8 (U+200B) character marker.
+     *
+     * @access public
+     * @param SyncFolder $folder
+     * @return boolean
+     */
+    public static function IsFolderToBeProcessedByKoe($folder) {
+        return isset($folder->displayname) && substr($folder->displayname, -3) == hex2bin("e2808b");
+    }
+
+    /**
+     * Check if the UTF-8 string has ISO-2022-JP esc seq
+     * if so, it is ISO-2022-JP, not UTF-8 and convert it into UTF-8
+     * string
+     *
+     * @access public
+     * @param $string
+     * @return $string
+     */
+    public static function CheckAndFixEncoding(&$string) {
+        if ( isset($string) && strpos($string, chr(0x1b).'$B') !== false ) {
+            $string = mb_convert_encoding($string, "utf-8", "ISO-2022-JP-MS");
+        }
+    }
 }
 
 
